/*
 * Title        :  DBConnection.java
 * Package      :  org.gridbus.gridscape.core.data
 * Project      :  Gridscape Core
 * Description	:  		
 * Created on   :  6/12/2004
 * Modified on  :  Feb 9, 2006 by Hussein Gibbins for use in Gridscape
 * Author		:  Krishna Nadiminti (kna@unimelb.edu.au)
 * Copyright    :  (c) 2006, Grid Computing and Distributed Systems Laboratory, 
 * 					   Dept. of Computer Science and Software Engineering,
 * 					   University of Melbourne, Australia.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later  version.
 * See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
 * 
 */

package org.gridbus.gridscape.core.data;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.Properties;

/**
 * Provides a database connection.
 * 
 */
public class DBConnection {

	//default assumes mysql database on localhost, with username "brokeruser" and pwd "gridbus"
	private  String drivername="";
	private  String connString="";
	private  String uname="";
	private  String pwd="";
	private  Connection conn=null;

	/**
	 */
	public DBConnection() {
        loadConfig();
	}
	
	/**
	 * @param driver
	 * @param connectionString
	 * @param username
	 * @param password
	 * @throws Exception
	 */
    public DBConnection(String driver,String connectionString,String username,String password) throws Exception {
		drivername = driver;
		connString = connectionString;
		uname = username;
		pwd = password;
	}
   
	/**
	 * @return a Connection object
	 * @throws InstantiationException 
	 * @throws IllegalAccessException 
	 * @throws ClassNotFoundException 
	 * @throws SQLException 
	 */
	public Connection getConnection() throws InstantiationException, IllegalAccessException, ClassNotFoundException, SQLException {
        if (drivername!=null){
            //register the driver with the DriverManager
            Class.forName(drivername).newInstance();
        }
		//connect if there is not connection already
        if (conn==null || conn.isClosed()){
			conn=DriverManager.getConnection(connString,uname,pwd);
		}
		return conn;
	}
	
    /**
     * Configures the properties
     */
    private void loadConfig() {
        Properties props = new Properties();
        try {
            //look for the Broker.properties config file.
            props.load(new FileInputStream(new File("GridscapeDB.properties")));
        } catch (Exception fe) {
            //if it cannot be loaded, try the CLASSPATH method which is more portable
            try {
                InputStream is = null;
                try{ //try in the resources directory
                    is = this.getClass().getResourceAsStream("resources/GridscapeDB.properties");
                }catch (Exception e1) {}
                
                if (is==null){ //try in the CLASSPATH 
                    is = this.getClass().getResourceAsStream("/GridscapeDB.properties");
                }
                props.load(is);
            } catch (Exception e) {
            }
        }
        
        this.drivername = props.getProperty("DRIVER");
        this.connString = props.getProperty("CONNECTION_STRING");
        this.uname = props.getProperty("USER");
        this.pwd = props.getProperty("PWD");
    }
    
	/**
	 * @throws Exception
	 */
	public void closeConnection() throws Exception{
		if (conn!=null){
			if (!conn.isClosed()) conn.close();
			conn=null;
		}
	}
	
	/**
	 * @return Returns the connectionString.
	 */
	public String getConnectionString() {
		return connString;
	}
	/**
	 * @param connectionString The connectionString to set.
	 */
	public void setConnectionString(String connectionString) {
		connString = connectionString;
	}
	/**
	 * @return Returns the password.
	 */
	public String getPassword() {
		return pwd;
	}
	/**
	 * @param passwd The password to set.
	 */
	public void setPassword(String passwd) {
		pwd = passwd;
	}
	/**
	 * @return Returns the username.
	 */
	public  String getUsername() {
		return uname;
	}
	/**
	 * @param username The username to set.
	 */
	public void setUsername(String username) {
		uname = username;
	}
	/**
	 * @return Returns the driver.
	 */
	public String getDriver() {
		return drivername;
	}
	/**
	 * @param driver The driver to set.
	 */
	public void setDriver(String driver) {
		drivername = driver;
	}
}
