/*
 * Title        :  DBQuerier.java
 * Package      :  org.gridbus.gridscape.core.data
 * Project      :  Gridscape Core
 * Description	:  		
 * Created on   :  Feb 9, 2005
 * Author		:  Hussein Gibbins (hag@cs.mu.oz.au)
 * Copyright    :  (c) 2006, Grid Computing and Distributed Systems Laboratory, 
 * 					   Dept. of Computer Science and Software Engineering,
 * 					   University of Melbourne, Australia.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later  version.
 * See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
 * 
 */

package org.gridbus.gridscape.core.data;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.LinkedHashMap;
import java.util.Vector;

import org.gridbus.gridscape.core.AppConfig;
import org.gridbus.gridscape.core.Country;
import org.gridbus.gridscape.core.Location;
import org.gridbus.gridscape.core.Resource;
import org.gridbus.gridscape.core.ResourceInfo;

/**
 * Abstraction layer for calls to the database.
 * 
 * @author hag
 *
 */
public class DBQuerier {

	private DBConnection dbc=null;
	private Connection conn=null;

	/**
	 * @throws Exception
	 */
	public DBQuerier() throws Exception {
		this.dbc = new DBConnection();
		connect();
	}
	/**
	 * @param dbc
	 * @throws Exception
	 * 
	 */
	public DBQuerier(DBConnection dbc) throws Exception {
		this.dbc = dbc;
		connect();
	}

	/**
	 * Constructor specifying the JDBC driver class name to use
	 * @param driver
	 * @param connectionString
	 * @param username
	 * @param password
	 * @throws Exception
	 */
	public DBQuerier(String driver,String connectionString,String username,String password) throws Exception{
        dbc = new DBConnection();
        dbc.setDriver(driver);
        dbc.setConnectionString(connectionString);
        dbc.setUsername(username);
        dbc.setPassword(password);
		connect();
	}

	/**
	 * Sets up the database connection
	 * @throws Exception
	 */
	public void connect() throws Exception{
		conn = dbc.getConnection();
	}

	/**
	 * Closes the database connection
	 * @throws Exception
	 */
	public void disconnect() throws Exception{
		dbc.closeConnection();
	}
    
    /**
     * @return all resources
     */
    public LinkedHashMap getResources() {
        return getResources(null);
    }
    
    /**
     * @param query
     * @return resources matching the query
     */
    public LinkedHashMap getResources(String query) {
        LinkedHashMap resources = new LinkedHashMap();
        Resource r = null;
        try {
            String search = "";
            PreparedStatement ps = null;
            if (query != null) {
                search = "SELECT id,hostname,description,type,online FROM resource,info " +
                            " WHERE ((resource.hostname LIKE '%"+query+"%' " + 
                            " OR resource.description LIKE '%"+query+"%' " +
                            " OR resource.type LIKE '%"+query+"%' " +
                            " OR info.info LIKE '%"+query+"%') " + 
                            " AND resource.id=info.resource_id)";
//                ps.setString(1,query);
//                ps.setString(2,query);
//                ps.setString(3,query);
//                ps.setString(4,query);
            } else {
                search = "SELECT id,hostname,description,type,online FROM resource";
            }

            ps = conn.prepareStatement(search);
            
            ResultSet rs = ps.executeQuery();
            while (rs.next()){
                r = new Resource();
                r.setID(rs.getInt("id"));
                r.setHostname(rs.getString("hostname"));
                r.setDescription(rs.getString("description"));
                r.setType(rs.getString("type"));
                r.setOnline(rs.getBoolean("online"));
                r.setLocation(getLocation(r.getID()));
                r.setInfo(getResourceInfo(r.getID()));
                resources.put(Integer.toString(r.getID()),r);
            }
            rs.close();
            ps.close();

        } catch (Exception e) {
            e.printStackTrace();
            // do nothing.
        }
        return resources;
    }
    
    /**
     * @param id
     * @return info for specified resource
     */
    public ResourceInfo getResourceInfo(int id) {
        ResourceInfo info = null;
        try {
            PreparedStatement ps = conn.prepareStatement("SELECT info FROM info WHERE resource_id=?");
            ps.setInt(1,id);
            ResultSet rs = ps.executeQuery();
            if (rs.next()){
                info = new ResourceInfo();
                info.setInfo(rs.getString("info"));
            }
            rs.close();
            ps.close();
            
        } catch (Exception e) {
            e.printStackTrace();
            // do nothing.
        }
        return info;
    }

    /**
     * @param id
     * @return location for specified resource
     */
    public Location getLocation(int id) {
        Location l = null;
        try {
            PreparedStatement ps = conn.prepareStatement("SELECT latitude,longitude,country_id FROM location WHERE resource_id=?");
            ps.setInt(1,id);
            ResultSet rs = ps.executeQuery();
            if (rs.next()){
                l = new Location();
                l.setLatitude(rs.getString("latitude"));
                l.setLongitude(rs.getString("longitude"));
                l.setCountry(getCountry(rs.getInt("country_id")));
            }
            rs.close();
            ps.close();

        } catch (Exception e) {
            e.printStackTrace();
            // do nothing.
        }
        return l;
    }

    /**
     * @param id
     * @return country
     */
    public Country getCountry(int id) {
        Country c = null;
        try {
            PreparedStatement ps = conn.prepareStatement("SELECT id,name,code FROM country WHERE id=?");
            ps.setInt(1,id);
            ResultSet rs = ps.executeQuery();
            if (rs.next()){
                c = new Country();
                c.setID(rs.getInt("id"));
                c.setName(rs.getString("name"));
                c.setCode(rs.getString("code"));
            }
            rs.close();
            ps.close();

        } catch (Exception e) {
            e.printStackTrace();
            // do nothing.
        }
        return c;
    }

    /**
     * @param id
     * @param r
     */
    public void updateResource(int id, Resource r) {

        try {
            PreparedStatement ps = null;
            
            ps = conn.prepareStatement("UPDATE resource SET hostname=?,description=?,type=? WHERE id=?");
            ps.setString(1,r.getHostname());
            ps.setString(2,r.getDescription());
            ps.setString(3,r.getType());
            ps.setInt(4,id);
            ps.execute();
            ps.close();

            ps = conn.prepareStatement("UPDATE location SET latitude=?,longitude=?,country_id=? WHERE resource_id=?");
            ps.setString(1,r.getLocation().getLatitude());
            ps.setString(2,r.getLocation().getLongitude());
            ps.setInt(3,r.getLocation().getCountry().getID());
            ps.setInt(4,id);
            ps.execute();
            ps.close();
            
        } catch (Exception e) {
            e.printStackTrace();
            // do nothing.
        }
    }

    /**
     * @param id
     * @param info
     */
    public void updateResourceInfo(int id, ResourceInfo info) {

        try {
            PreparedStatement ps = conn.prepareStatement("UPDATE info SET info=? WHERE resource_id=?");
            ps.setString(1,info.getInfo());
            ps.setInt(2,id);
            ps.execute();
            ps.close();

        } catch (Exception e) {
            e.printStackTrace();
            // do nothing.
        }
    }

    /**
     * @param id
     * @param isOnline
     */
    public void setOnline(int id, boolean isOnline) {

        try {
            PreparedStatement ps1 = conn.prepareStatement("UPDATE resource SET online=? WHERE id=?");
            ps1.setBoolean(1,isOnline);
            ps1.setInt(2,id);
            ps1.execute();
            ps1.close();
            
        } catch (Exception e) {
            e.printStackTrace();
            // do nothing.
        }
    }
    
    /**
     * @param r
     * @return the id of the created resource
     */
    public int createResource(Resource r) {

        int id = -1;
        
        try {
            PreparedStatement ps = null;

            ps = conn.prepareStatement("INSERT INTO resource(hostname,description,type) VALUES(?,?,?)",PreparedStatement.RETURN_GENERATED_KEYS);
            ps.setString(1,r.getHostname());
            ps.setString(2,r.getDescription());
            ps.setString(3,r.getType());
            ps.execute();
            ResultSet resultset = ps.getGeneratedKeys();
            if (resultset.next()) {
                id = resultset.getInt(1);
            } else {
                // if we didnt create the resource just return.
                return -1;
            }
            ps.close();

            Location l = r.getLocation();
            ps = conn.prepareStatement("INSERT INTO location(resource_id,latitude,longitude,country_id) VALUES(?,?,?,?)");
            ps.setInt(1,id);
            ps.setString(2,l.getLatitude());
            ps.setString(3,l.getLongitude());
            ps.setInt(4,l.getCountry().getID());
            ps.execute();
            ps.close();

            ps = conn.prepareStatement("INSERT INTO info(resource_id) VALUES(?)");
            ps.setInt(1,id);
            ps.execute();
            ps.close();

        } catch (Exception e) {
            e.printStackTrace();
            // do nothing.
        }
        
        return id;
    }

    /**
     * @param id
     */
    public void deleteResource(int id) {
        
        try {
            PreparedStatement ps = null;

            ps = conn.prepareStatement("DELETE FROM resource WHERE id=?");
            ps.setInt(1,id);
            ps.execute();
            ps.close();

            ps = conn.prepareStatement("DELETE FROM info WHERE resource_id=?");
            ps.setInt(1,id);
            ps.execute();
            ps.close();

            ps = conn.prepareStatement("DELETE FROM location WHERE resource_id=?");
            ps.setInt(1,id);
            ps.execute();
            ps.close();

        } catch (Exception e) {
            e.printStackTrace();
            // do nothing.
        }
    }
    
    /**
     * @return list of available countries
     */
    public Vector getCountries() {
        Vector countries = new Vector();
        Country c = null;
        try {
            PreparedStatement ps = conn.prepareStatement("SELECT id,name,code FROM country");
            ResultSet rs = ps.executeQuery();
            while (rs.next()){
                c = new Country();
                c.setID(rs.getInt("id"));
                c.setName(rs.getString("name"));
                c.setCode(rs.getString("code"));
                countries.add(c);
            }
            rs.close();
            ps.close();
            
        } catch (Exception e) {
            e.printStackTrace();
            // do nothing.
        }
        return countries;
    }
    
    /**
     * @return list of available countries
     */
    public AppConfig getAppConfig() {
        AppConfig config = new AppConfig();
        try {
            PreparedStatement ps = conn.prepareStatement("SELECT mapKey,dimH,dimW,lat,lng,mapType,allowZoom,showToolbar,showList,zoomLevel,baseURL FROM appconfig WHERE id=1");
            
            ResultSet rs = ps.executeQuery();
            if (rs.next()){
                config = new AppConfig();
                config.setMapKey(rs.getString("mapKey"));
                config.setDimH(rs.getString("dimH"));
                config.setDimW(rs.getString("dimW"));
                config.setLat(rs.getString("lat"));
                config.setLng(rs.getString("lng"));
                config.setMapType(rs.getString("mapType"));
                config.setAllowZoom(rs.getBoolean("allowZoom"));
                config.setShowToolbar(rs.getBoolean("showToolbar"));
                config.setShowList(rs.getBoolean("showList"));
                config.setZoomLevel(rs.getInt("zoomLevel"));
                config.setBaseURL(rs.getString("baseURL"));
            }
            rs.close();
            ps.close();

        } catch (Exception e) {
            e.printStackTrace();
            // do nothing.
        }
        return config;
    }
    
    /**
     * @param a 
     * 
     */
    public void updateAppConfig(AppConfig a) {

        try {
            PreparedStatement ps = null;
            ps = conn.prepareStatement("UPDATE appconfig SET mapKey=?,baseURL=?,dimH=?,dimW=?,lat=?,lng=?,zoomLevel=?,mapType=?"+
                                                            ",allowZoom=?,showToolbar=?,showList=?");
            ps.setString(1,a.getMapKey());
            ps.setString(2,a.getBaseURL());
            ps.setString(3,a.getDimH());
            ps.setString(4,a.getDimW());
            ps.setString(5,a.getLat());
            ps.setString(6,a.getLng());
            ps.setInt(7,a.getZoomLevel());
            ps.setString(8,a.getMapType());
            ps.setBoolean(9,a.isAllowZoom());
            ps.setBoolean(10,a.isShowToolbar());
            ps.setBoolean(11,a.isShowList());
            ps.execute();
            ps.close();
        } catch (Exception e) {
            e.printStackTrace();
            // do nothing.
        }
    }
}
