/*
 * Title        :  GridscapeEdit.java
 * Package      :  org.gridbus.gridscape.portlets
 * Project      :  Gridscape Portal
 * Description  :       
 * Created on   :  Feb 9, 2006
 * Author       :  Hussein Gibbins (hag@cs.mu.oz.au)
 * Copyright    :  (c) 2006, Grid Computing and Distributed Systems Laboratory, 
 *                     Dept. of Computer Science and Software Engineering,
 *                     University of Melbourne, Australia.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later  version.
 * See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
 * 
 */

package org.gridbus.gridscape.portlets;

import java.io.IOException;
import java.util.LinkedHashMap;
import java.util.Vector;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletException;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.gridbus.gridscape.core.AppConfig;
import org.gridbus.gridscape.core.Country;
import org.gridbus.gridscape.core.Location;
import org.gridbus.gridscape.core.Resource;
import org.gridbus.gridscape.core.data.DBQuerier;

/**
 * Portlet for editing resources.
 * 
 * @author hag
 */
public class GridscapeEdit extends GenericPortlet {

	/**
	 * @param request
	 * @param response
	 * @throws PortletException
	 * @throws IOException
	 */
    public void processAction(ActionRequest request, ActionResponse response) throws PortletException, IOException {

        String form = request.getParameter("form");
        String option = request.getParameter("option");
        DBQuerier querier = null;
        try {
            querier = new DBQuerier();
            
            if (form.equalsIgnoreCase("customise")) {
                // process customise form
                AppConfig appcfg = new AppConfig();
                appcfg.setMapKey(request.getParameter("mapKey"));
                appcfg.setBaseURL(request.getParameter("baseURL"));
                appcfg.setDimH(request.getParameter("dimH"));
                appcfg.setDimW(request.getParameter("dimW"));
                appcfg.setLat(request.getParameter("lat"));
                appcfg.setLng(request.getParameter("lng"));
                appcfg.setZoomLevel(Integer.parseInt(request.getParameter("zoomLevel")));
                appcfg.setMapType(request.getParameter("mapType"));
                appcfg.setAllowZoom(Boolean.valueOf(request.getParameter("allowZoom")).booleanValue());
                appcfg.setShowToolbar(Boolean.valueOf(request.getParameter("showToolbar")).booleanValue());
                appcfg.setShowList(Boolean.valueOf(request.getParameter("showList")).booleanValue());

                querier.updateAppConfig(appcfg);

            } else if (form.equalsIgnoreCase("resource")) {
                // process resource form
                
                String selectedId = null;
                
                if (option.equalsIgnoreCase("edit")) {
                    // EDIT: edit selected resource details.
                    selectedId = request.getParameter("resource");
                    // select resource for editing
                    if (selectedId == null) {
                        selectedId = "-1";
                    }
                    response.setRenderParameter("selectedId", selectedId);
                } else if (option.equalsIgnoreCase("cancel")) {
                    // CANCEL: cancel edit changes for selected resource
                    response.setRenderParameter("selectedId", "-1");
                } else if (option.equalsIgnoreCase("new")) {
                    // NEW: create new resource
                    // add new record to db and return its id (just select the hostname back out again
                    // selected id should be the id of the newly inserted record
                    String hostname = request.getParameter("newHostname");
                    Resource r = new Resource();
                    r.setHostname(hostname);
                    r.setDescription("");
                    r.setType("GT2");
                    
                    Location l = new Location();
                    l.setLatitude("0");
                    l.setLongitude("0");
                    l.setCountry(new Country());
                    
                    r.setLocation(l);
                    
                    int id = querier.createResource(r);
                    
                    response.setRenderParameter("selectedId", Integer.toString(id));
                    
                } else if (option.equalsIgnoreCase("ok, accept changes")) {
                    // OK: accept edit changes for selected resource
                    // accept changes and clear selected resource
                    // get long and lat and write to db
                    selectedId = request.getParameter("selectedId");
                    // general properties
                    String hostname = request.getParameter("hostname");
                    String description = request.getParameter("description");
                    String type = request.getParameter("type");
                    // location properties
                    String country = request.getParameter("country");
                    String latitude = request.getParameter("latitude");
                    String longitude = request.getParameter("longitude");
        
                    Resource r = new Resource();
                    r.setHostname(hostname);
                    r.setDescription(description);
                    r.setType(type);
        
                    Location l = new Location();
                    l.setLatitude(latitude);
                    l.setLongitude(longitude);
        
                    Country c = new Country();
                    c.setID(Integer.parseInt(country));
        
                    l.setCountry(c);
                    r.setLocation(l);
                    
                    querier.updateResource(Integer.parseInt(selectedId), r);
        
                    response.setRenderParameter("selectedId", "-1");
                } else if (option.equalsIgnoreCase("delete")) {
                    // DELETE: delete the selected resource
                    selectedId = request.getParameter("selectedId");
                    querier.deleteResource(Integer.parseInt(selectedId));
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                querier.disconnect();
            } catch (Exception e) {}
        }
    }

    /**
     * @param request
     * @param response
     * @throws PortletException
     * @throws IOException
     */
    public void doView(RenderRequest request, RenderResponse response) throws PortletException, IOException {
        
        // put into init
        DBQuerier querier = null;
        LinkedHashMap resources = null;
        Vector countries = null;
        AppConfig appcfg = null;
        
        String selectedId = request.getParameter("selectedId");
        if (selectedId == null) {
            // don't select anything
            selectedId = "-1";
        }
        
        try {
            // put into init
            querier = new DBQuerier();
            appcfg = querier.getAppConfig();
            resources = querier.getResources();
            countries = querier.getCountries();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                querier.disconnect();
            } catch (Exception e) {}
        }
        
        request.setAttribute("appcfg", appcfg);        
        request.setAttribute("url", response.createActionURL().toString());
        request.setAttribute("selectedId", selectedId);
        request.setAttribute("resources",resources);
        request.setAttribute("countries",countries);
        getPortletContext().getRequestDispatcher("/jsp/gridscape_edit.jsp").include(request, response);
    }    
}
