/*
 * Title        :  StyleUtil.java
 * Package      :  org.gridbus.gridscape.util
 * Project      :  Gridscape Portal
 * Description	:  		
 * Created on   :  Feb 16, 2006
 * Author		:  Hussein Gibbins (hag@cs.mu.oz.au)
 * Copyright    :  (c) 2006, Grid Computing and Distributed Systems Laboratory, 
 * 					   Dept. of Computer Science and Software Engineering,
 * 					   University of Melbourne, Australia.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later  version.
 * See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
 * 
 */

package org.gridbus.gridscape.util;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.net.URL;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.gridbus.gridscape.core.AppConfig;
import org.gridbus.gridscape.core.data.DBQuerier;
import org.gridbus.gridscape.core.display.Displayable;

/**
 * Simple util used to perform XSL transforms using object xml and stylesheets.
 * 
 * The Style mechanism is used in order to give Gridscape a lot of flexibility in
 * terms of what it can display.  This class takes the XML representation of a display
 * object to display (ie. Resource or ResourceInfo objects) and some format parameters
 * such as resource type and the context in which the information is to be displayed.
 * It then picks up the correct xsl stylesheet (if one exists) and uses that to format
 * the object to be displayed.
 * 
 * Using this principle, the display can be changed easily without any change to the
 * codebase, simply by adding a new stylesheet.  If a new resource type becomes available
 * a stylesheet simply needs to be created to format the information.
 * 
 * @author hag
 *
 */
public class StyleUtil {

    /**
     * Specifies a resource info element is being formatted
     */
    public static final String ELEMENT_INFO = "info";
    /**
     * Specifies a resource element is being formatted
     */
    public static final String ELEMENT_RESOURCE = "resource";
    /**
     * Specifies that the information should be formatted for use in a row context
     */
    public static final String FORMAT_ROW = "row";
    /**
     * Specifies that the information should be formatted for use in a popup context
     */
    public static final String FORMAT_POPUP = "popup";
    
    /**
     * Performs XSL transformation on object XML and XSL stylesheet
     * 
     * @param obj
     * @param type
     * @param element
     * @param format
     * @return formatted html
     */
    public static String style(Displayable obj,String type,String element,String format) {
        String output = "";//xml;
        ByteArrayOutputStream os = new ByteArrayOutputStream();
        try {
            DBQuerier querier = new DBQuerier();
            AppConfig appcfg = querier.getAppConfig();
            
            URL url = new URL (appcfg.getBaseURL()+"/xsl/"+(type == null ? "":type+"_")+element+"_"+format+".xsl");
            String data = obj.toXML();
            
            if (!data.trim().equals("")) {
                TransformerFactory tFactory = TransformerFactory.newInstance();
                Transformer transformer = tFactory.newTransformer(new StreamSource(url.openStream()));
                transformer.transform(
                        new StreamSource(new ByteArrayInputStream(data.getBytes())), 
                        new StreamResult(os));
                output = os.toString();
            }
        } catch (Exception e) {
            e.printStackTrace();
//            return xml;
        }
        output = output.replaceAll("\r","");
        output = output.replaceAll("\n","");
        return output;
    }
    
// would like a method that just takes an object and knew how to format it
//    public String style(Object o) {
//        
//    }

    
}
