/*
 * Title        :  GT2InfoGatherer.java
 * Package      :  org.gridbus.gridscape.resourcemonitor.plugins
 * Project      :  Gridscape Resource Monitor
 * Description	:  		
 * Created on   :  Feb 9, 2006
 * Author		:  Hussein Gibbins (hag@cs.mu.oz.au)
 * Copyright    :  (c) 2006, Grid Computing and Distributed Systems Laboratory, 
 * 					   Dept. of Computer Science and Software Engineering,
 * 					   University of Melbourne, Australia.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later  version.
 * See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
 * 
 */

package org.gridbus.gridscape.resourcemonitor.plugins;

import javax.naming.NamingEnumeration;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.SearchResult;

import org.gridbus.broker.farming.globus.MDSUtil;
import org.gridbus.gridscape.core.Resource;
import org.gridbus.gridscape.core.ResourceInfo;
import org.gridbus.gridscape.resourcemonitor.InfoGatherer;

/**
 * Globus 2.x Info Gatherer.  Queries MDS to discover resource information.
 * 
 * @author hag
 *
 */
public class GT2InfoGatherer implements InfoGatherer {
    
    /**
     * @return info
     */
    public ResourceInfo gatherInfo(Resource r) {
        ResourceInfo info = new ResourceInfo();
        String xml = "";
        
        xml += "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>";
        xml += "<info>";
        
        String[] HOST_ATTRIBUTES = {
            "Mds-Computer-isa",
            "Mds-Cpu-model",
            "Mds-Cpu-speedMHz",
            "Mds-Cpu-Total-count",
            "Mds-Cpu-version",
            "Mds-Fs-freeMB",
            "Mds-Memory-Ram-Total-freeMB",
            "Mds-Memory-Ram-Total-sizeMB",
            "Mds-Cpu-Total-Free-1minX100",
            "Mds-Cpu-Total-Free-5minX100",
            "Mds-Cpu-Total-Free-15minX100",
            "Mds-Os-name",
            "Mds-Net-addr"};
       
        try { //TODO need to do something about this: MDS hangs sometimes!
            String filter = "(&(objectclass=MdsHost)(Mds-Host-hn="+r.getHostname()+"))";
            NamingEnumeration results=MDSUtil.search("ldap://"+r.getHostname()+":2135", filter, HOST_ATTRIBUTES);
            
            if(results==null){
                System.err.println("setValues() - Error in accessing MDS!! Host not present");
            }else{              
                while(results.hasMore()){
                    SearchResult sr=(SearchResult)results.next();
                    Attributes attrs=sr.getAttributes();
                    for (NamingEnumeration e = attrs.getAll(); e.hasMore();)
                    {
                        // BUILD XML DOC HERE //
                        Attribute temp=(Attribute)e.next();
                        String value="";
                        for(NamingEnumeration ne=temp.getAll();ne.hasMore();value=value.concat((String)ne.next()));
                        
                        xml += "<attribute><name>"+temp.getID()+"</name><value>"+value+"</value></attribute>";

                    }
                }
            }
        
        } catch (Exception e){
            System.err.println("Exception. Couldnot get information from MDS on remote host "+r.getHostname());
            System.err.println(e.getMessage());
            return null;
        }
        
        xml += "</info>";
        
        info.setInfo(xml);
        
        return info;
    }
}
