/*
 * Title        :  GT2InfoGatherer.java
 * Package      :  org.gridbus.gridscape.resourcemonitor.plugins
 * Project      :  Gridscape Resource Monitor
 * Description	:  		
 * Created on   :  Feb 9, 2006
 * Author		:  Hussein Gibbins (hag@cs.mu.oz.au)
 * Copyright    :  (c) 2006, Grid Computing and Distributed Systems Laboratory, 
 * 					   Dept. of Computer Science and Software Engineering,
 * 					   University of Melbourne, Australia.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later  version.
 * See the GNU General Public License (http://www.gnu.org/copyleft/gpl.html)for more details.
 * 
 */

package org.gridbus.gridscape.resourcemonitor.plugins;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.net.URL;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.globus.axis.util.Util;
import org.globus.wsrf.WSRFConstants;
import org.globus.wsrf.utils.AnyHelper;
import org.gridbus.gridscape.core.Resource;
import org.gridbus.gridscape.core.ResourceInfo;
import org.gridbus.gridscape.resourcemonitor.InfoGatherer;
import org.oasis.wsrf.properties.QueryExpressionType;
import org.oasis.wsrf.properties.QueryResourcePropertiesResponse;
import org.oasis.wsrf.properties.QueryResourceProperties_Element;
import org.oasis.wsrf.properties.QueryResourceProperties_PortType;
import org.oasis.wsrf.properties.WSResourcePropertiesServiceAddressingLocator;

/**
 * Globus 4 Info Gatherer.  Queries MDS4 via web services interface to discover
 * services available.
 * 
 * @author hag
 *
 */
public class GT4InfoGatherer implements InfoGatherer {
    
    /*
     * Required for axis https transport registration on the client side.
     */
    static{
        Util.registerTransport();
    }
    
    /**
     * @param r
     * @return info
     */
    public ResourceInfo gatherInfo(Resource r) {

        String xml = "";
//        String xmlHeader = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>"+"</info>";
//        String xmlFooter = "</info>";
        
        ResourceInfo info = new ResourceInfo();

        String dialect = WSRFConstants.XPATH_1_DIALECT;
        String xpathQuery = "//*[local-name()='MemberServiceEPR' or local-name()='ComputingElement']";

        WSResourcePropertiesServiceAddressingLocator locator =
            new WSResourcePropertiesServiceAddressingLocator();

        try {
            QueryExpressionType query = new QueryExpressionType();
            query.setDialect(dialect);
            query.setValue(xpathQuery);//expression);
            
            // query the default information service
            QueryResourceProperties_PortType port =
            locator.getQueryResourcePropertiesPort(new URL("https://"+r.getHostname()+":8443/wsrf/services/DefaultIndexService"));
            
            QueryResourceProperties_Element request = new QueryResourceProperties_Element();
            request.setQueryExpression(query);
            QueryResourcePropertiesResponse response = port.queryResourceProperties(request);
            
            xml = AnyHelper.toSingleString(response);

            xml = "<info>"+xml+"</info>";
            
        } catch(Exception e) {
            System.err.println("Exception. Couldnot get information from MDS4 on remote host "+r.getHostname());
            System.err.println(e.getMessage());
            return null;
        }
        
        
        // reformat xml into something cleaner so that presentation doesnt need to work so hard.
        String classname = this.getClass().getName();
        classname = classname.substring(classname.lastIndexOf(".")+1);

        URL url = this.getClass().getResource("/"+classname+".xsl");        

        ByteArrayOutputStream os = new ByteArrayOutputStream();
        try {
            String xslFilePath = url.getPath();
            File xslFile = new File(xslFilePath);

            TransformerFactory tFactory = TransformerFactory.newInstance();
            Transformer transformer = tFactory.newTransformer(new StreamSource(xslFile));
            transformer.transform(
                    new StreamSource(new ByteArrayInputStream(xml.getBytes())), 
                    new StreamResult(os));
            xml = os.toString();
        } catch (Exception e) {
            e.printStackTrace();
        }        
        
        info.setInfo(xml);

        return info;
    }
}
